%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definitions of builtins of module IOExts:
%

:- use_module('../prologbasics').
:- use_module('../basics').


% create a new INET socket:
?- block socketINET(?,?,-,?).
socketINET(_,'$io'(Socket),E0,E) :-
        newSocket(Socket),
        E0=E.


% bind an INET socket to a port:
?- block prim_socketBind(?,?,?,?,-,?).
prim_socketBind(RSocket,RPort,_,'$io'('Prelude.()'),E0,E) :-
	deref(RSocket,Socket),
	deref(RPort,Port),
        socketBind(Socket,_Host,Port),
        E0=E.


% set backlog queue of a socket:
?- block prim_socketListen(?,?,?,?,-,?).
prim_socketListen(RSocket,RLength,_,'$io'('Prelude.()'),E0,E) :-
	deref(RSocket,Socket),
	deref(RLength,Length),
        socketListen(Socket,Length), !,
        E0=E.


% return the first connection to a socket as a read/write stream:
?- block prim_socketAccept(?,?,?,-,?).
prim_socketAccept(RSocket,_,
		  '$io'('Prelude.(,)'(ClientS,
			      '$stream'('$inoutstream'(InStream,OutStream)))),
		  E0,E) :-
	deref(RSocket,Socket),
        socketAccept(Socket,Client,InStream,OutStream),
	atom2String(Client,ClientS), !,
        E0=E.

% return a connection to a socket within a time limit as a read/write stream,
% otherwise Nothing:
?- block prim_waitForSocketAccept(?,?,?,?,-,?).
prim_waitForSocketAccept(RSocket,RTO,_,'$io'(Result),E0,E) :-
	deref(RSocket,Socket),
	deref(RTO,TimeOut),
	(TimeOut<0 -> TO=off
	            ; TOSec is TimeOut//1000,
	              TOMSec is (TimeOut mod 1000) * 1000, % in milliseconds
	              TO = TOSec:TOMSec),
	(waitForSocketClientStream(Socket,TO,Client,InStream,OutStream)
	 -> atom2String(Client,ClientS),
	    Result = 'Prelude.Just'('Prelude.(,)'(ClientS,
			      '$stream'('$inoutstream'(InStream,OutStream))))
	  ; Result = 'Prelude.Nothing'),
        E0=E.


% open a connection to a Unix socket:
?- block prim_connectToSocket(?,?,?,?,-,?).
prim_connectToSocket(RSHst,RSNr,_,
		     '$io'('$stream'('$inoutstream'(InStream,OutStream))),
		     E0,E) :-
	derefAll(RSHst,SHst), deref(RSNr,SNr),
        string2Atom(SHst,Host), !,
        newSocket(Socket),
        socketConnect(Socket,Host,SNr,InStream,OutStream), E0=E.


